require 'rspec'
require 'yaml'
require_relative '../lib/determinations_comparison'


describe 'determinations_comparison' do

  let(:baseline_file) do
    File.join('spec','support','data','test_batch','file.po')
  end

  let(:undertest_file) do
    File.join('spec','support','data','test_batch','file.json')
  end

  let(:baseline_within_thresholds_file) do
    File.join('spec','support','data','test_batch','file_within_thresholds.po')
  end

  let(:undertest_within_thresholds_file) do
    File.join('spec','support','data','test_batch','file_within_thresholds.json')
  end

  let(:baseline_diffwoscale_file) do
    File.join('spec','support','data','test_batch','file_withdiff_woscale.po')
  end

  let(:undertest_diffwoscale_file) do
    File.join('spec','support','data','test_batch','file_withdiff_woscale.json')
  end


  let(:folderpath_output) do
    File.join('bin')
  end

  let(:folderpath_with_log) do
    File.join('spec','support','data','test_batch')
  end

  before :each do
    FileUtils.rm(Dir.glob('bin/*'))
    FileUtils.rm(Dir.glob('bin2/*'))
  end
  
  it 'should show differences as hash' do
    comparison = DeterminationsComparison::Comparison.new( baseline_file, undertest_file, {:folderpath_output=>folderpath_output})

    hash = comparison.as_hash
    expect(hash.size).to eq(2)
  end

  it 'should categorize determination' do
    comparison = DeterminationsComparison::Comparison.new( baseline_file, undertest_file, {:folderpath_output=>folderpath_output})

    expect(comparison.hashCategory[:desc]).to eq('peak picked by baseline but not by undertest')
  end

  it 'should show differences in html table and peak plots' do
    comparison = DeterminationsComparison::Comparison.new( baseline_file, undertest_file)

    retcode = comparison.to_html folderpath_output
    expect(retcode).to eq(true)
    expect(File.exists?(comparison.html_filepath)).to eq(true)
    expect(File.exists?(comparison.png_filepath)).to eq(true)
  end

  it 'should show differences in html table and peak plots (all differences within thresholds)' do
    comparison = DeterminationsComparison::Comparison.new( baseline_within_thresholds_file, undertest_within_thresholds_file)

    retcode = comparison.to_html folderpath_output
    expect(retcode).to eq(true)
    expect(File.exists?(comparison.html_filepath)).to eq(true)
    expect(File.exists?(comparison.png_filepath)).to eq(true)
  end

  it 'should show differences in html table and peak plots (with differences not requiring scale)' do
    comparison = DeterminationsComparison::Comparison.new( baseline_diffwoscale_file, undertest_diffwoscale_file)

    retcode = comparison.to_html folderpath_output
    expect(retcode).to eq(true)
    expect(File.exists?(comparison.html_filepath)).to eq(true)
    expect(File.exists?(comparison.png_filepath)).to eq(true)
  end


  it 'should show not show peak plot but still show diffs in table if problem running CompareDeterminations' do
    comparison = DeterminationsComparison::Comparison.new( baseline_file, undertest_file, {:filepath_comparedeterminations=>'/bogus/path/CompareDetermiatinos.py'})

    retcode = comparison.to_html folderpath_output
    expect(retcode).to eq(true)
    expect(File.exists?(comparison.html_filepath)).to eq(true)
    expect(File.exists?(comparison.png_filepath)).to eq(false)
  end

  it 'should allow user to specify different location for log.txt file' do
    folderpath_with_log_alternative = File.join('spec','support','data')
    comparison = DeterminationsComparison::Comparison.new( baseline_file, undertest_file, {:folderpath_with_log=>folderpath_with_log_alternative})
    hash = comparison.as_hash
    expect(hash.size).to eq(2)
  end

  it 'should use alternative folderpath' do
    comparison = DeterminationsComparison::Comparison.new( baseline_file, undertest_file)
    hash = comparison.to_html 'bin2'
    expect(Dir.glob('bin2/*').size).to_not eq(0)
  end

end