require 'rspec'
require_relative '../lib/html_table_of_property_differences'

describe 'generating html table of differences' do

  let(:html_table) do
    arrItems =  [{:chromatogram=>'quant', :property=>'expected_retention_time', :diff=>0.0, :baseline=>18.808, :undertest=>18.808, :calculation=>'(((18.808 - 18.808.abs / scale ) * 100).round(2)', :scale=>'relative_to_peak_start_and_end_time (-1.191998-38.808002)'}, {:chromatogram=>'quant', :property=>'peak_index', :diff=>100.0, :baseline=>7, :undertest=>0, :calculation=>'0 - 7 / 7', :scale=>''}, {:chromatogram=>'quant', :property=>'start_index', :diff=>100.0, :baseline=>114.0, :undertest=>0, :calculation=>'0 - 114.0 / 114.0', :scale=>''}, {:chromatogram=>'quant', :property=>'start_time', :diff=>42.5, :baseline=>17.0, :undertest=>0, :calculation=>'(((17.0 - 0.abs / scale ) * 100).round(2)', :scale=>'relative_to_peak_start_and_end_time (-1.191998-38.808002)'}, {:chromatogram=>'quant', :property=>'start_intensity', :diff=>72.91, :baseline=>13683.7, :undertest=>0, :calculation=>'(((13683.7 - 0.abs / scale ) * 100).round(2)', :scale=>'relative_to_apex_intensity (0-18767.7659496771)'}, {:chromatogram=>'quant', :property=>'start_base_int', :diff=>100.0, :baseline=>10960.0, :undertest=>0, :calculation=>'0 - 10960.0 / 10960.0', :scale=>''}, {:chromatogram=>'quant', :property=>'end_index', :diff=>100.0, :baseline=>136.0, :undertest=>0, :calculation=>'0 - 136.0 / 136.0', :scale=>''}, {:chromatogram=>'quant', :property=>'end_time', :diff=>100.0, :baseline=>20.3, :undertest=>0, :calculation=>'0 - 20.3 / 20.3', :scale=>''}, {:chromatogram=>'quant', :property=>'end_intensity', :diff=>61.45, :baseline=>11533.4, :undertest=>0, :calculation=>'(((11533.4 - 0.abs / scale ) * 100).round(2)', :scale=>'relative_to_apex_intensity (0-18767.7659496771)'}, {:chromatogram=>'quant', :property=>'end_base_int', :diff=>100.0, :baseline=>10309.5, :undertest=>0, :calculation=>'0 - 10309.5 / 10309.5', :scale=>''}, {:chromatogram=>'quant', :property=>'apex_index', :diff=>100.0, :baseline=>123.0, :undertest=>0, :calculation=>'0 - 123.0 / 123.0', :scale=>''}, {:chromatogram=>'quant', :property=>'apex_time', :diff=>100.0, :baseline=>18.4, :undertest=>0, :calculation=>'0 - 18.4 / 18.4', :scale=>''}, {:chromatogram=>'quant', :property=>'apex_intensity', :diff=>100.0, :baseline=>18767.8, :undertest=>0, :calculation=>'0 - 18767.8 / 18767.8', :scale=>''}, {:chromatogram=>'quant', :property=>'baseline_intercept', :diff=>100.0, :baseline=>14301.1, :undertest=>0, :calculation=>'0 - 14301.1 / 14301.1', :scale=>''}, {:chromatogram=>'quant', :property=>'baseline_slope', :diff=>100.0, :baseline=>-196.5, :undertest=>0, :calculation=>'0 - -196.5 / -196.5', :scale=>''}, {:chromatogram=>'quant', :property=>'height', :diff=>100.0, :baseline=>8096.1, :undertest=>0, :calculation=>'0 - 8096.1 / 8096.1', :scale=>''}, {:chromatogram=>'quant', :property=>'area', :diff=>100.0, :baseline=>13710.5, :undertest=>0, :calculation=>'0 - 13710.5 / 13710.5', :scale=>''}, {:chromatogram=>'quant', :property=>'prob', :diff=>100.0, :baseline=>1, :undertest=>0, :calculation=>'0 - 1 / 1', :scale=>''}, {:chromatogram=>'quant', :property=>'nls_amp', :diff=>100.0, :baseline=>11499.0, :undertest=>0, :calculation=>'0 - 11499.0 / 11499.0', :scale=>''}, {:chromatogram=>'quant', :property=>'nls_mu', :diff=>100.0, :baseline=>18.0, :undertest=>0, :calculation=>'0 - 18.0 / 18.0', :scale=>''}, {:chromatogram=>'quant', :property=>'nls_sig', :diff=>100.0, :baseline=>0.5, :undertest=>0, :calculation=>'0 - 0.5 / 0.5', :scale=>''}, {:chromatogram=>'quant', :property=>'nls_tau', :diff=>100.0, :baseline=>1.5, :undertest=>0, :calculation=>'0 - 1.5 / 1.5', :scale=>''}, {:chromatogram=>'quant', :property=>'fitted_start', :diff=>41.25, :baseline=>16.5, :undertest=>0, :calculation=>'(((16.5 - 0.abs / scale ) * 100).round(2)', :scale=>'relative_to_peak_start_and_end_time (-1.191998-38.808002)'}, {:chromatogram=>'quant', :property=>'fitted_end', :diff=>57.5, :baseline=>23.0, :undertest=>0, :calculation=>'(((23.0 - 0.abs / scale ) * 100).round(2)', :scale=>'relative_to_peak_start_and_end_time (-1.191998-38.808002)'}, {:chromatogram=>'quant', :property=>'fitted_coverage', :diff=>100.0, :baseline=>0.8, :undertest=>0, :calculation=>'0 - 0.8 / 0.8', :scale=>''}, {:chromatogram=>'quant', :property=>'smooth_signal_to_noise_ratio', :diff=>100.0, :baseline=>5, :undertest=>0, :calculation=>'0 - 5 / 5', :scale=>''}, {:chromatogram=>'quant', :property=>'filter_tsz', :diff=>0.0, :baseline=>10, :undertest=>10, :calculation=>'10 - 10 / 10', :scale=>''}, {:chromatogram=>'is', :property=>'expected_retention_time', :diff=>0.0, :baseline=>18.958, :undertest=>18.958, :calculation=>'(((18.958 - 18.958.abs / scale ) * 100).round(2)', :scale=>'relative_to_peak_start_and_end_time (8.958002-28.958002)'}, {:chromatogram=>'is', :property=>'peak_index', :diff=>0.0, :baseline=>1, :undertest=>1, :calculation=>'1 - 1 / 1', :scale=>''}, {:chromatogram=>'is', :property=>'start_index', :diff=>0.0, :baseline=>107, :undertest=>107, :calculation=>'107 - 107 / 107', :scale=>''}, {:chromatogram=>'is', :property=>'start_time', :diff=>0.0, :baseline=>15.95, :undertest=>15.95, :calculation=>'(((15.95 - 15.95.abs / scale ) * 100).round(2)', :scale=>'relative_to_peak_start_and_end_time (8.958002-28.958002)'}, {:chromatogram=>'is', :property=>'start_intensity', :diff=>0.02, :baseline=>1060.46, :undertest=>1268.27, :calculation=>'(((1060.46 - 1268.27.abs / scale ) * 100).round(2)', :scale=>'relative_to_apex_intensity (0-1031938.23030771)'}, {:chromatogram=>'is', :property=>'start_base_int', :diff=>4.55, :baseline=>1281.4, :undertest=>1342.44, :calculation=>'1281.4 - 1342.44 / 1342.44', :scale=>''}, {:chromatogram=>'is', :property=>'end_index', :diff=>0.0, :baseline=>166, :undertest=>166, :calculation=>'166 - 166 / 166', :scale=>''}, {:chromatogram=>'is', :property=>'end_time', :diff=>0.0, :baseline=>24.827, :undertest=>24.827, :calculation=>'24.827 - 24.827 / 24.827', :scale=>''}, {:chromatogram=>'is', :property=>'end_intensity', :diff=>0.01, :baseline=>3860.63, :undertest=>3808.17, :calculation=>'(((3860.63 - 3808.17.abs / scale ) * 100).round(2)', :scale=>'relative_to_apex_intensity (0-1031938.23030771)'}, {:chromatogram=>'is', :property=>'end_base_int', :diff=>3.28, :baseline=>1446.92, :undertest=>1496.01, :calculation=>'1446.92 - 1496.01 / 1496.01', :scale=>''}, {:chromatogram=>'is', :property=>'apex_index', :diff=>0.0, :baseline=>126, :undertest=>126, :calculation=>'126 - 126 / 126', :scale=>''}, {:chromatogram=>'is', :property=>'apex_time', :diff=>0.0, :baseline=>18.808, :undertest=>18.808, :calculation=>'18.808 - 18.808 / 18.808', :scale=>''}, {:chromatogram=>'is', :property=>'apex_intensity', :diff=>0.03, :baseline=>1031938.2, :undertest=>1031680.0, :calculation=>'1031938.2 - 1031680.0 / 1031680.0', :scale=>''}, {:chromatogram=>'is', :property=>'baseline_intercept', :diff=>7.74, :baseline=>984.0, :undertest=>1066.5, :calculation=>'984.0 - 1066.5 / 1066.5', :scale=>''}, {:chromatogram=>'is', :property=>'baseline_slope', :diff=>7.78, :baseline=>18.6458, :undertest=>17.2998, :calculation=>'18.6458 - 17.2998 / 17.2998', :scale=>''}, {:chromatogram=>'is', :property=>'height', :diff=>0.02, :baseline=>1038944.4, :undertest=>1038760.0, :calculation=>'1038944.4 - 1038760.0 / 1038760.0', :scale=>''}, {:chromatogram=>'is', :property=>'area', :diff=>0.02, :baseline=>2009217.4, :undertest=>2009700.0, :calculation=>'2009217.4 - 2009700.0 / 2009700.0', :scale=>''}, {:chromatogram=>'is', :property=>'prob', :diff=>0.0, :baseline=>1, :undertest=>1, :calculation=>'1 - 1 / 1', :scale=>''}, {:chromatogram=>'is', :property=>'nls_amp', :diff=>0.05, :baseline=>1319908.0, :undertest=>1319260.0, :calculation=>'1319908.0 - 1319260.0 / 1319260.0', :scale=>''}, {:chromatogram=>'is', :property=>'nls_mu', :diff=>0.0, :baseline=>18.3704, :undertest=>18.3709, :calculation=>'18.3704 - 18.3709 / 18.3709', :scale=>''}, {:chromatogram=>'is', :property=>'nls_sig', :diff=>0.07, :baseline=>0.607286, :undertest=>0.607729, :calculation=>'0.607286 - 0.607729 / 0.607729', :scale=>''}, {:chromatogram=>'is', :property=>'nls_tau', :diff=>0.03, :baseline=>1.66762, :undertest=>1.66815, :calculation=>'1.66762 - 1.66815 / 1.66815', :scale=>''}, {:chromatogram=>'is', :property=>'fitted_start', :diff=>0.0, :baseline=>16.4064, :undertest=>16.4064, :calculation=>'(((16.4064 - 16.4064.abs / scale ) * 100).round(2)', :scale=>'relative_to_peak_start_and_end_time (8.958002-28.958002)'}, {:chromatogram=>'is', :property=>'fitted_end', :diff=>0.0, :baseline=>23.3497, :undertest=>23.3497, :calculation=>'(((23.3497 - 23.3497.abs / scale ) * 100).round(2)', :scale=>'relative_to_peak_start_and_end_time (8.958002-28.958002)'}, {:chromatogram=>'is', :property=>'fitted_coverage', :diff=>0.0, :baseline=>1, :undertest=>1, :calculation=>'1 - 1 / 1', :scale=>''}, {:chromatogram=>'is', :property=>'smooth_signal_to_noise_ratio', :diff=>99.16, :baseline=>3, :undertest=>356.4, :calculation=>'3 - 356.4 / 356.4', :scale=>''}, {:chromatogram=>'is', :property=>'filter_tsz', :diff=>0.0, :baseline=>10, :undertest=>10, :calculation=>'10 - 10 / 10', :scale=>''}]
    HTML_Table_of_Property_Differences.new(arrItems)
  end

  let(:file) do
    File.join('htmltabletest.html')
  end

  it 'should generate html table' do
    html_table.render
    html_table.save file
  end

  after :each do
    FileUtils.remove file
  end

end